<?PHP
/*
 * XML Converter : Convert a FoxyCart XML datafeed into CSV or QuickBooks IIF files
 * Copyright (C) 2008 Jeremy Saye
 * Distributed under the GPL License.
 *
 * Developed for RenewableChoice.com:
 * Be a "green" web professional and offset your carbon usage at www.renewablechoice.com.
 *
 * Title: 
 * 	config.php
 * 
 * Description:
 * 	The configuration file for the XML converter script. All options and settings 
 * 	are managed here.
 * 
 * Author:
 * 	Jeremy Saye (software@jeremysaye.com)
 * 	
 * Version:
 * 	0.2.0, 2008-03-11
 */

// ==================================================================
// General Options
// ==================================================================

// Set your key to decrypt the XML that you receive from FoxyCart.
// This must match *exactly* what you entered at http://www.foxycart.com/admin
$XML_KEY = 'CHANGE THIS TEXT to your own datafeed keyphrase';

// The path to the directory where the downloadable files are located.
// The download script (via apache) MUST have permission to write to this directory!
// This path must end with a slash: '/'
$FILE_PATH = '/home/domains/example.com/web/files/';

// What should the files be called? The current date and file extension will be appended.
$FILE_IDENTIFIER = 'foxycart';

// The url for the location of the convert script.
// This should be like http://SERVER_URL/PATH-FROM-WEB-ROOT-TO-CURRENT-DIRECTORY/convertXML.php
$SCRIPT_URL = 'http://www.example.com/download/convertXML.php';

// Datafeeds may be posted every five minutes, and the converted output can be captured to a file that rotates as often as you like. 
// Specify one of the following options below: MONTH, WEEK, DAY, or HOUR
$FILE_CREATE_FREQUENCY = 'WEEK';

// ==================================================================
// Communication options
// ==================================================================

// The email address to send errors and download confirmation messages to.
$ADMIN_EMAIL = 'admin@example.com';

// The subject line for the confirmation message.
$MESSAGE_SUBJECT = 'FoxyCart IIF file ready';

// ==================================================================
// File output options
// ==================================================================

// CSV, IIF, XML
$OUTPUT_FORMAT = 'IIF';

// Separator character(s)
$FIELD_SEPARATOR = "\t"; 	// Typically a tab: "\t" or comma
$LINE_SEPARATOR = "\r\n";	// Typically a carriage return and newline: "\r\n"

/* 
 * Here's where the real power lies.
 * The following three sections allow you to define exactly how your CSV or IIF file will be laid out. The first section opens the file and has the main order/customer data. The second section is for each individual item in the transaction (such as if the customer purchased multiple items at once). The final section is for the file footer, if necessary.
 *
 * Think of the final file as a spreadsheet. Each line defines a single column in the file; the lines are comma-separated lists of values.
 * <HEADER>, <FOXYCART LOCATION=>FIELD>, <CONSTANT VALUE>, <PROPERTY (CUSTOM_CODE)>
 *
 * <HEADER> - the value to put in the header line for this column at the top of the file
 *
 * <FOXYCART LOCATION> - indicates where in the XML datafeed this value can be located.
 * 	Plain fieldnames are assumed to be in the main 'transaction' section of the feed. Any other location must be specified with the following format: LOCATION=>FIELD_NAME
 * 	The following are all valid sections that can be referenced in this way:
 * 		transaction
 * 		custom_field
 * 		transaction_detail
 * 		transaction_detail_option
 * 		
 * <CONSTANT VALUE> - is dual purpose. If no location was specified, the value here will be used instead. However, you can define a 'format' so that this value can be dynamically generated based on other fields present in the datafeed. Specify them with the field name in double brackets, like this: [[customer_first_name]] At this time only fields in the main 'transaction' section can be referenced here. You can intermix your substitutions with other characters.
 * 	Example: 
 * 		[[customer_last_name]]::[[transaction_date]]
 * 		would result in:
 * 		Doe::2007-05-04 20:53:57
 *
 * <PROPERTY> - special handling. If you put 'CUSTOM_CODE' here, the value for the field can be derived from whatever custom code is defined in the appropriate section at the end of this config file.
 *
 * A number sign (#) can be placed at the start of any line to "comment" it out so it is ignored.
 *
 * Commas are reserved characters, so escape them with a backslash '\' to use them literally.
 *
 * Do NOT remove the 'EOT;' line - that marks the end of each section!
 */

$FORMAT_ORDER = <<<EOT
# You can modify, but don't 
!TRNS, , TRNS

TRNSID, id
TRNSTYPE, , INVOICE
Account, , , CUSTOM_CODE
Invoice, , , CUSTOM_CODE
PurchaseDate, transaction_date
StartDate, transaction_date
Rep, , WS
Terms, , Monthly-CC
JobStatus, , InProgress
JobType, , Residential
Class, , , CUSTOM_CODE
Amount, order_total
CustName, , [[customer_first_name]] [[customer_last_name]]
FirstName, customer_first_name
LastName, customer_last_name
NameOnCard, , [[customer_first_name]] [[customer_last_name]]
Email, customer_email
# Enter other fields here as necessary (this line will be ignored)...
Phone, customer_phone
EOT;
// end of ORDER section

// items - there may be several of these per order
$FORMAT_ITEM = <<<EOT
!SPL, , SPL
ElectricitykWh, transaction_detail_option=>electricitykwh
MonthlyPrice, transaction_detail=>product_price
Item, transaction_detail=>product_name
Quantity, transaction_detail=>product_quantity
Rate, transaction_detail_option=>rate
Amount, , , CUSTOM_CODE
OrderType, transaction_detail_option=>order_type
PromoSource, transaction_detail_option=>promo_source
Location, , 0
EOT;
// end of ITEM section

// Close out this entry of the file with a custom footer.
$FORMAT_FOOTER = <<<EOT
!ENDTRNS, , ENDTRNS
EOT;
// end of FOOTER section

// =========================================================================
// Define CUSTOM code in this section to get the value for each situation.
// =========================================================================

/*
 * The below is simply PHP code, and the following data is available:
 * header_field 	=> the name of the field to currently find the value for. 
 * 				Matches <HEADER> for the appropriate row in the config area above.
 * current_data 	=> the current data object to process
 * current_section 	=> the name of the section of the XML we're currently inside
				This can be used to differentiate if the $header_field is ambiguous.
 * transaction 		=> the root-level transaction object
 *
 * Getting data out of each parsed-XML object ($current_data and $transaction) can be confusing at first.
 * Specific fields are referenced as follows: $transaction->{FIELD_NAME}[0]->tagData
 * So, to get the data inside the date:
	<transaction>
		<id>616</id>
		<transaction_date>2007-05-04 20:53:57</transaction_date>
		...
	</transaction>
 * Do: $value = $transaction->{transaction_date}[0]->tagData;
 *
 * To get the data for the second transaction detail option when $current_data contains the transaction_detail:
	<transaction_detail>
		<product_name>foo</product_name>
		<product_price>20.00</product_price>
		<transaction_detail_options>
			<transaction_detail_option>
				<product_option_name>color</product_option_name>
				<product_option_value>blue</product_option_value>
			</transaction_detail_option>
			<transaction_detail_option>
				<product_option_name>color</product_option_name>
				<product_option_value>RED-GET-THIS-ONE</product_option_value>
			</transaction_detail_option>
		</transaction_detail_options>
	</transaction_detail>
 * Do: $value = $current_data->{transaction_detail_options}[0]->{transaction_detail_option}[1]->{product_option_value}[0]->tagData;
 *
 * Consult the documentation inside include/class.xmlparser_php5.php for further information.
 */

function get_custom_value($header_field, $current_data, $current_section='transaction', $transaction)
{
	$value = '';

	// =========================================================================
	// Place your custom code inside 'if' and 'elseif' blocks below.
	// =========================================================================

	if ($header_field == 'Account')
	{
		// [LastName],[FirstName][Incremental 3 digit number]
		// Example: Last,First-000
		$value = $transaction->{customer_last_name}[0]->tagData . ',';
		$value .= $transaction->{customer_first_name}[0]->tagData . '-';
		// Force the customer_id to 3 characters.
		$value .= sprintf("%03.3s", $transaction->{customer_id}[0]->tagData);
	}

	elseif ($header_field == 'Invoice')
	{
		// [Purchase date minus slashes][First & Last Initial][Incremental 2 digit number]
		// Example: 012908SP54

		// Get the date without slashes and without the time at the end.
		$value = str_replace('-', '', $transaction->{transaction_date}[0]->tagData);
		$value = preg_replace("/\s*[\d:]*$/", "", $value);

		$value .= substr($transaction->{customer_first_name}[0]->tagData, 0, 1);
		$value .= substr($transaction->{customer_last_name}[0]->tagData, 0, 1);
		// Force the customer_id to 2 characters.
		$value .= sprintf("%02.2s", $transaction->{customer_id}[0]->tagData);
	}

	elseif ($header_field == 'Class')
	{
		// [JobType]-[Initials of Reward product selected]
		$value = 'Residential' . '-';
		$value .= substr($transaction->{reward_product}[0]->tagData, 0, 1);
		// TODO: find where the reward product is defined and get its initials
	}

	elseif ($header_field == 'Amount' && $current_section == 'transaction_detail')
	{
		// Subscription products - determined by product attribute
		if (isset($current_data->{subscription_frequency})
			&& $current_data->{subscription_frequency}[0]->tagData != '')
		{
			// Annual price (per 12 months)
			$value = ($current_data->{product_price}[0]->tagData * 12);
		}
		else
		{
			// Non-subscription products.
			$value = $current_data->{product_price}[0]->tagData;
		}
	}
	
	// =========================================================================
	// End of custom code
	// =========================================================================

	return $value;
}
// end of function get_custom_value

// This is the end of the configuration file.

?>