<?PHP
/*
 * XML Converter : Convert a FoxyCart XML datafeed into CSV or QuickBooks IIF files
 * Copyright (C) 2008 Jeremy Saye
 * Distributed under the GPL License.
 *
 * Developed for RenewableChoice.com:
 * Be a "green" web professional and offset your carbon usage at www.renewablechoice.com.
 *
 * Title: 
 * 	convertXML.php
 * 
 * Description:
 * 	Script to process FoxyCart-provided encrypted XML, then output a file in some
 * 	specified format, from QuickBooks IIF to plain CSV.
 * 
 * Author:
 * 	Jeremy Saye (software@jeremysaye.com)
 * 	
 * Version:
 * 	0.2.0, 2008-03-11
 */

// Set the following variable to 'TRUE' if you want to enable debugging output for testing. 
// Please note that FoxyCart will reject such output as an error.
$debug = FALSE;

// ======================================================================================
// Do not modify after this line.
// ======================================================================================

// Include the necessary files.
require_once('config.php');

$output = '';

// Process the configuration file.
$config_order = process_config($FORMAT_ORDER);
$config_item = process_config($FORMAT_ITEM);
$config_footer = process_config($FORMAT_FOOTER);

// Combine the configs to allow for easy header generation.
$config_combined = array_merge($config_order, $config_item, $config_footer);

// Define what the valid sections of the foxycart XML are.
$valid_fc_sections = array(
	'transaction'			=> 'transactions',
	'custom_field' 			=> 'custom_fields',
	'transaction_detail' 		=> 'transaction_details',
	'transaction_detail_option' 	=> 'transaction_detail_options',
	'shipto_address' 		=> 'shipto_addresses',
	'shipto_custom_field' 		=> 'shipto_custom_fields',
);

// See if we were posted to.
if (!isset($_POST['FoxyData']))
{
	// If you're here, you didn't receive the FoxyData post, so error out.
	// This ensures that the transactions will still show up on the next datafeed.
	print "Error: no post data\n";
	exit;
}

// Decrypt the data using your $key
require_once('include/class.rc4crypt.php');
// Then decrypt the XML
$FoxyData_encrypted = urldecode($_POST['FoxyData']);
$FoxyData_decrypted = rc4crypt::decrypt($XML_KEY,$FoxyData_encrypted);

// Parse the XML.
// We'll be using Adam Flynn's XMLParser so we can use the same code for both php4 and php5.
// http://www.criticaldevelopment.net/xml/doc.php
// the chunk of code below is figuring that out for you.
if (PHP_VERSION < 5)
{
	require_once('include/class.xmlparser_php4.php');
	//Set up the parser object
	$data = new XMLParser($FoxyData_decrypted);
	$data->Parse();
}
else if (PHP_VERSION >= 5)
{
	// NOTE: PHP5 has XML parsing built in with SimpleXML.
	// If you're going to be building your own script, you might want to use that instead:
	// $data = new SimpleXMLElement($FoxyData_decrypted);
	
	// Otherwise, we'll be using Adam A Flynn's XMLParser (like we're using for PHP4 above).
	require_once('include/class.xmlparser_php5.php');
	//Set up the parser object
	$data = new XMLParser($FoxyData_decrypted);
	$data->Parse();
}

// The XML is now an array called $data.
if (is_object($data))
{
	// Each line for the file will be stored in an array.
	$file_arr = array();

	// TODO: Do not continue if no data was posted (FC is not supposed to do this..)

	// Build the file name.
	// The date determines how often a new file will be created. (defaults to daily)
	$date_string = 'Ymd';
	if (strtoupper($FILE_CREATE_FREQUENCY) == 'MONTH')
	{
		$date_string = 'Ym';
	}
	elseif (strtoupper($FILE_CREATE_FREQUENCY) == 'WEEK')
	{
		// Use a more legible format for this.
		$date_string = 'Y\weekWM';
	}
	elseif (strtoupper($FILE_CREATE_FREQUENCY) == 'HOUR')
	{
		$date_string = 'Ymd-H';
	}

	$date = date($date_string);

	$ext = strtolower($OUTPUT_FORMAT);
	$filename = $FILE_IDENTIFIER.'_'.$date.'.'.$ext;
	$path = $FILE_PATH.$filename;

	// Make sure the directory is writable.
	if (!is_writable($FILE_PATH))
	{
		$output = "Error: $FILE_PATH not writable!\n";
		send_error_email($output);
		if ($debug) { print "$output\n"; }
		exit;
	}

	// Process formats other than raw XML.
	if ($OUTPUT_FORMAT != 'XML')
	{
		// Generate the header line, but only if writing to a new file.
		if (!file_exists($path))
		{
			$header_line = '';
			foreach ($config_combined as $subarray)
			{
				// IIF headers start a new line if they begin with '!'
				if ($OUTPUT_FORMAT == 'IIF' && preg_match("/^!/", $subarray[0]))
				{
					array_push($file_arr, $header_line);
					$header_line = '';
				}
				$header_line .= format_element($subarray[0]);
			}

			// Remove the final separator.
			$header_line = preg_replace("/$FIELD_SEPARATOR$/", "", $header_line);
			
			array_push($file_arr, $header_line);
		}

		// First, loop through the transactions
		foreach ($data->document->transactions[0]->transaction as $transaction)
		{
			if ($debug) { print "Process transaction $transaction_id\n"; }

			// Process a single order.
			$current_line = process_line($config_order, $transaction, 'transaction', $transaction);
			array_push($file_arr, $current_line);

			// Loop through the products per transaction and process each one.
			foreach ($transaction->transaction_details[0]->transaction_detail as $detail)
			{
				$current_line = process_line($config_item, $detail, 'transaction_detail', $transaction);
				array_push($file_arr, $current_line);
			}

			// Process the footer for this order.
			$current_line = process_line($config_footer, $transaction, 'transaction', $transaction);
			array_push($file_arr, $current_line);
		}
		// Done with all transactions/customers.
	}

	// Open the file for writing, append to the end, create if necessary.
	$handle = fopen($path, "a");

	if ($debug) { print "Output file $path\n\n"; }

	if ($OUTPUT_FORMAT == 'XML')
	{
		// Output the raw decrypted XML.
		fwrite($handle, $FoxyData_decrypted);
	}
	else
	{
		// Output the actual file based on the file array.
		foreach ($file_arr as $line)
		{
			$line .= $LINE_SEPARATOR;
			fwrite($handle, $line);
			
			if ($debug) { print $line; }
		}
	}

	// Close the handle.
	fclose($handle);
	// Return 'foxy'
	print 'foxy';
}
else
{
	// It's not an object, which means something went wrong (like an incorrect key).
	$output .= "Error: not an object - check your key.\n";
}

if ($output != '')
{
	// There was a problem with at least one transaction, so send an email.
	send_error_email($output);
}
else
{
	// Indicate success.
	$output = 'foxy';
}

if ($debug) { print "$output\n"; }
exit;

// ======================================================================================
// Helper functions.
// ======================================================================================

// Process a single 'FORMAT' section of the configuration file, parsing the values into an array.
function process_config($config_string)
{
	$final_config = array();
	$lines = explode("\n", $config_string);
	foreach ($lines as $line)
	{
		// Skip commented lines.
		if (preg_match("/^#/", $line))
		{
			continue;
		}

		// Replace escaped commas with something distinctive to avoid problems with the explode.
		$line = preg_replace("/\\\,/", "-::-COMMA-::-", $line);

		$line_arr = explode(",", $line);
		foreach ($line_arr as $key => $line_to_trim)
		{
			$line_to_trim = preg_replace("/-::-COMMA-::-/", ",", $line_to_trim);
			$line_arr[$key] = trim($line_to_trim);
		}

		array_push($final_config, $line_arr);
	}

	return $final_config;
}
// end of process_config

// Format a single element for IIF or CSV.
function format_element($elem)
{
	global $FIELD_SEPARATOR, $OUTPUT_FORMAT;
	if ($OUTPUT_FORMAT == 'IIF')
	{
		$final_elem = $elem . $FIELD_SEPARATOR;
		#if ($debug) { $final_elem = "'$elem'" . $FIELD_SEPARATOR; }
	}
	elseif ($OUTPUT_FORMAT == 'CSV')
	{
		if (preg_match("/\"|,|\n/", $elem))
		{
			// Use double quotes.
			$elem = preg_replace('/"/g', '""', $elem);
			$elem = preg_replace("/\A(.*)\z/gms", "$1", $elem);
		}
		
		$final_elem = $elem . $FIELD_SEPARATOR;
	}
	
	// Note that the final separator MUST be removed if this is the last element in the line.
	// This must be performed by the calling function.

	return $final_elem;
}
// end of format_element

// Send an email message to the ADMIN_EMAIL address.
function send_error_email ($output)
{
	global $ADMIN_EMAIL;
	$subject = "FoxyCart XML conversion problem!!!";
	$body = "The following error occurred on " . date('Y-m-d H:i:s') . "\n$output";
	$headers = "From: $ADMIN_EMAIL\n";
	mail($ADMIN_EMAIL, $subject, $body, $headers);
}
// end of send_error_email

// Process a single line.
function process_line($config=array(), $current_data, $current_section='transaction', $transaction)
{
	// We always want access to the main transaction data, even if we're looking at a detail now.
	global $valid_fc_sections, $debug, $FIELD_SEPARATOR;

	// Generate the transaction line.
	$current_line = '';
	foreach ($config as $subarray)
	{
		list($header_field, $fc_field, $constant_value, $custom) = $subarray;

		if (!isset($header_field))
		{
			continue;
		}

		// Determine the value to use for this field.
		$value = NULL;

		if (isset($custom) && $custom == 'CUSTOM_CODE')
		{
			// Use the custom code inside config.inc.php to get the value for this field.
			$value = get_custom_value($header_field, $current_data, $current_section, $transaction);
		}
		elseif (isset($constant_value))
		{
			// Constant value or pattern.
			if (preg_match_all("/(\[\[[^]]+\]\])/", $constant_value, $matches))
			{
				// Process the pattern.
				$value = $constant_value;
				$replacement = '';
				// Get the value for each match.
				foreach ($matches[1] as $match)
				{
					// Strip off the brackets and figure out the replacement value.
					$fieldname = preg_replace("/^\[\[(.*)\]\]$/", "$1", $match);
					if (array_key_exists($fieldname, $current_data))
					{
						$replacement = $current_data->{$fieldname}[0]->tagData;
					}
					else
					{
						print "Error: could not find match for $fieldname\n";
					}

					// Apply the new value.
					$value = preg_replace("/\[\[[^]]+\]\]/", $replacement, $value, 1);
				}
			}
			else
			{
				// Use the constant value.
				$value = $constant_value;
			}
		}
		elseif (isset($fc_field))
		{
			if (preg_match("/^(.*)=>(.*)$/", $fc_field, $matches) 
				&& !preg_match("/^transaction=>/", $fc_field)
			)
			{
				// Make sure this section is accessible.
				list($full_match, $fc_section, $fc_field) = $matches;

				if (!array_key_exists($fc_section, $valid_fc_sections))
				{
					print "Error: $fc_section does not exist in the XML\n";
					continue;
				}

				// Go to that area of the xml.
				if ($fc_section == $current_section)
				{
					// Get the current transaction_detail.
					$value = $current_data->{$fc_field}[0]->tagData;
				}

				elseif ($fc_section == 'transaction_detail')
				{
					// Get the first one.
					$value = $transaction->{transaction_details}[0]->{transaction_detail}[0]->{$fc_field}[0]->tagData;
				}

				elseif ($current_section == 'transaction_detail' && $fc_section == 'transaction_detail_option')
				{
					// Find the correct option of the current detail.
					foreach ($current_data->{transaction_detail_options}[0]->{transaction_detail_option} as $option)
					{
						if ($option->{product_option_name}[0]->tagData == $fc_field)
						{
							$value = $option->{product_option_value}[0]->tagData;
							break;
						}
					}
					if ($value == '')
					{
						print "Error: custom field '$fc_field' does not exist in the XML\n";
						continue;
					}
				}

				elseif ($fc_section == 'custom_field')
				{
					// Find a field with a matching name.
					foreach ($transaction->{custom_fields}[0]->{custom_field} as $custom_field)
					{
						if ($custom_field->{custom_field_name}[0]->tagData == $fc_field)
						{
							$value = $custom_field->{custom_field_value}[0]->tagData;
							break;
						}
					}
					if ($value == '')
					{
						print "Error: custom field '$fc_field' does not exist in the XML\n";
						continue;
					}
				}

				elseif ($fc_section == 'shipto_address')
				{
					// TODO: add handling for this, and shipto_custom_field.
				}

				else
				{
					// Cannot get to the requested field from here.
					print "Error: $fc_section => $fc_field does not exist in the XML\n";
				}
			}
			else
			{
				// Find the value in the main (transaction) data.
				if (array_key_exists($fc_field, $transaction))
				{
					$value = $transaction->{$fc_field}[0]->tagData;
				}
				else
				{
					print "Error: field $fc_field not found!!\n";
					continue;
				}
			}
		}
		else
		{
			// No clue what to do with this element.
			$output .= "Error: no behavior defined for $header_field\n";
		}

		// Format each element for CSV or whatever.
		$current_line .= format_element($value);
	}

	// Remove the final separator.
	$current_line = preg_replace("/$FIELD_SEPARATOR$/", "", $current_line);

	return $current_line;
}
// end of process_line

?>
