<?PHP
/*
 * XML Converter : Convert a FoxyCart XML datafeed into CSV or QuickBooks IIF files
 * Copyright (C) 2008 Jeremy Saye
 * Distributed under the GPL License.
 *
 * Developed for RenewableChoice.com:
 * Be a "green" web professional and offset your carbon usage at www.renewablechoice.com.
 *
 * Title: 
 * 	nofity.php
 * 
 * Description:
 * 	Script to check on a regular basis if a new IIF/CSV file has been generated.
 * 	If so, an email notification is sent to the admin user with a download link.
 * 	For example, if the files are created weekly, the script will check on Monday 
 * 	(the start of a new week) for the presence of a completed file. Monthly files 
 * 	are checked on the first of a new month.
 *
 * Usage:
 * 	Call this file from a cron job on a daily basis.
 * 
 * Author:
 * 	Jeremy Saye (software@jeremysaye.com)
 * 	
 * Version:
 * 	0.2.0, 2008-03-11
 */

// Include the necessary files. If this file has been moved to another directory, make sure that the below is still correctly referencing the location of your config file!
require_once('config.php');

// Check if a new file has recently been written.

// Build the file name.
// The date determines how often a new file will be created
$date_string = '';
$date_arr = localtime(time(), TRUE);
if (strtoupper($FILE_CREATE_FREQUENCY) == 'MONTH')
{
	// Is this the first day of the month?
	if (date('j') != 1) { exit; }

	$date_string = 'Ym';
	$date_arr['tm_mon']--;
}
elseif (strtoupper($FILE_CREATE_FREQUENCY) == 'WEEK')
{
	// Did a week just roll over? Look for monday (Sunday is 0).
	if (date('w') != 1) { exit; }
	
	$date_string = 'Y_\weekW_M';
	$date_arr['tm_mday'] -= 7;
}
elseif (strtoupper($FILE_CREATE_FREQUENCY) == 'DAY')
{
	$date_string = 'Ymd';
	$date_arr['tm_mday']--;
}
elseif (strtoupper($FILE_CREATE_FREQUENCY) == 'HOUR')
{
	// This just gets the file created in the last hour, not every file created since yesterday.
	$date_string = 'Ymd-H';
	$date_arr['tm_hour']--;
}

// Construct the timestamp.
$prev_date = mktime($date_arr['tm_hour'], $date_arr['tm_min'], $date_arr['tm_sec'], $date_arr['tm_mon'], $date_arr['tm_mday'], $date_arr['tm_year'], $date_arr['tm_isdst']);

// Construct the filename.
$ext = strtolower($OUTPUT_FORMAT);
$date = date($date_string, $prev_date);
$path = $FILE_PATH.$FILE_IDENTIFIER.'_'.$date.'.'.$ext;

#print "Look for file: $path\n";

if (file_exists($path))
{
	print "found file\n"; exit;

	// Send an email with a download link.
	$body = "FoxyCart XML datafeed file available\n\n";
	$body .= "Download the file here:\n";
	$body .= $SCRIPT_URL . $filename . "\n";

	$headers = "From: $ADMIN_EMAIL\n";
	mail($ADMIN_EMAIL, $MESSAGE_SUBJECT, $body, $headers);
}
else
{
	print "No file found.";
	print "<br /> Current Date:" . date('Y-m-d');
	print "<br /> tm_mon:" . $date_arr['tm_mon'];
	print "<br /> tm_mday:" . $date_arr['tm_mon'];
	print "<br /> tm_hour:" . $date_arr['tm_mon'];
	print "<br /> tm_year:" . $date_arr['tm_mon'];
	print "<br /> Date:" . $date;
	print "<br /> Path:" . $path;
	print "<br /> Filename:" . $filename;
}

exit;

?>
